cmake_minimum_required(VERSION 3.16..3.25)

set(libbinio_known_comps static shared)
set(libbinio_comp_static NO)
set(libbinio_comp_shared NO)
foreach (libbinio_comp IN LISTS ${CMAKE_FIND_PACKAGE_NAME}_FIND_COMPONENTS)
    if (libbinio_comp IN_LIST libbinio_known_comps)
        set(libbinio_comp_${libbinio_comp} YES)
    else ()
        set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
            "libbinio does not recognize component `${libbinio_comp}`.")
        set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
        return()
    endif ()
endforeach ()

if (libbinio_comp_static AND libbinio_comp_shared)
    set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
        "libbinio `static` and `shared` components are mutually exclusive.")
    set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
    return()
endif ()

set(libbinio_static_targets "${CMAKE_CURRENT_LIST_DIR}/libbinio-static-targets.cmake")
set(libbinio_shared_targets "${CMAKE_CURRENT_LIST_DIR}/libbinio-shared-targets.cmake")

macro(libbinio_load_targets type)
    if (NOT EXISTS "${libbinio_${type}_targets}")
        set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
            "libbinio `${type}` libraries were requested but not found.")
        set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
        return()
    endif ()
    include("${libbinio_${type}_targets}")
endmacro()

if (libbinio_comp_static)
    libbinio_load_targets(static)
elseif (libbinio_comp_shared)
    libbinio_load_targets(shared)
elseif (DEFINED libbinio_SHARED_LIBS AND libbinio_SHARED_LIBS)
    libbinio_load_targets(shared)
elseif (DEFINED libbinio_SHARED_LIBS AND NOT libbinio_SHARED_LIBS)
    libbinio_load_targets(static)
elseif (BUILD_SHARED_LIBS)
    if (EXISTS "${libbinio_shared_targets}")
        libbinio_load_targets(shared)
    else ()
        libbinio_load_targets(static)
    endif ()
else ()
    if (EXISTS "${libbinio_static_targets}")
        libbinio_load_targets(static)
    else ()
        libbinio_load_targets(shared)
    endif ()
endif ()
